<?php

/**
* Checks if all server settings are ok
* for the wizadvisor
*
* @access public
* @return array
*/
function validate_server_settings()
{
    // init default value
	$settings = array();
		
	// check php version
	$settings['php_version'] = (!check_version(PHP_VERSION, '4.3.4')) ? false : true;
							  
	// check if php mysql extension is loaded
	$settings['mysql'] = (!extension_loaded('mysql')) ? false : true;

    // array of files wich must be writeable	
	$readwriteable_files = array('pst'                    => '../Connections/pst.php',
	                             'kt'                     => '../Connections/KT_db.php',
				                 'kt_frontend'            => '../Connections/KT_frontend_db.php',
								 'images_mid_high'        => '../img/Mid_High_resolution/product/',
								 'images_low'             => '../img/Low_resolution/product/',
								 'pdf'                    => '../pdf/product/',
								 'logo'                   => '../img/top2.gif',
								 'top'                    => '../img/top.gif',
                                 'tiny_mce_init'          => '../tinymce/init.js',
                                 'tiny_mce_ibrowser'      => '../tinymce/plugins/ibrowser/config/config.inc.php');
	
	// check all files if they are writeable
	foreach($readwriteable_files as $key => $file) {
	    if(!file_exists($file) || !is_writeable($file) || !is_readable($file)) {
			$settings[$key] = false;
		} else {
		    $settings[$key] = true;
		}
	}
	
	// now we go thru all settings and replace the true/false
	// value with an image
	foreach($settings as $key => $value) {
	    if($value) {
		    $settings[$key] = '<img src="good.gif" alt="false" width="12" height="15">';
		} else {
		    $settings[$key] = '<img src="bad.gif" alt="true" width="11" height="12">';
		}
	}
	
	return $settings;
}

/**
* Function to compare php versions
*
* @param string $currentversion
* @param string $requiredversion
* @access public
* @return bool
*/
function check_version($currentversion, $requiredversion) {
    list($majorC, $minorC, $editC) = split('[/.-]', $currentversion);
    list($majorR, $minorR, $editR) = split('[/.-]', $requiredversion);
  
    if ($majorC > $majorR) return true;
    if ($majorC < $majorR) return false;
    // same major - check ninor
    if ($minorC > $minorR) return true;
    if ($minorC < $minorR) return false;
    // and same minor
    if ($editC  >= $editR)  return true;
    return false;
}

/**
* Validates the Database settings form
*
* @param $settings
* @access public
* @return
*/
function validate_database_settings($settings)
{
    // init default value
	$errors   = array();
	
	// strip slashes if magic_quotes are enabled
	if(get_magic_quotes_gpc()) {
	    $settings = array_map('stripslashes', $settings);
	}
		
	// Check if Username is to short	
	if(!isset($settings['username']) || strlen($settings['username']) < 1) {
	    $errors[] = BAD_USERNAME;
	}
	
	// Check if hostname is to short	
	if(!isset($settings['hostname']) || strlen($settings['hostname']) < 1) {
	    $errors[] = BAD_HOSTNAME;
	}
	
	// Check if database is to short	
	if(!isset($settings['database']) || strlen($settings['database']) < 1) {
	    $errors[] = BAD_DATABASE;
	}
	
	// At this position we test if some errors
	// occured and if so we stop here and return those errors
    if(!empty($errors)) {
	    return $errors;
	}
	
	// Check if Database connection is possible with
	// this settings	
	$conn = @mysql_connect($settings['hostname'],
                           $settings['username'],
                           $settings['password']);
	if(!$conn) {
	    $errors[] = BAD_CONNECTION;
		return $errors;
	}
	
    // Check if Database exists
	if(!@mysql_select_db($settings['database'])) {
	    $errors[] = BAD_CONNECTION;
	}
	
    // Check if Databasefile exists and is in the right location
	$valid_files = glob('*.sql');
	if(!isset($settings['database_file']) ||
	   !file_exists($settings['database_file']) ||
	   !in_array($settings['database_file'], $valid_files)) {
	    $errors[] = DATABASE_FILE_404;
	}

    // return result
    return $errors;
}

/**
* Writes the MySQL Connection settings and the relative sitepath into the
* Connection files
*
* @param array $settings
* @access public
* @return array
*/
function write_database_settings($settings)
{
    // init default values
	$errors = array();
	
	// the path to the applications eg. http://yourdomain.com/instance/
    $app_path = dirname($_SERVER["SCRIPT_NAME"]);
    $app_path = str_replace('install', '', $app_path);
    
	// db settings to be written
    $db_settings = array('username'    => $settings['username'],
	                     'password'    => $settings['password'],
						 'hostname'    => $settings['hostname'],
						 'database'    => $settings['database']
                         );
	
	// connection files wich need to be modified
	$connection_files = array('pst'         => '../Connections/pst.php',
	                          'kt'          => '../Connections/KT_db.php',
				              'kt_frontend' => '../Connections/KT_frontend_db.php');		   

    $tinyMCE = array(
         array(
            'file'    => '../tinymce/init.js',
            'pattern' => '/document_base_url.*/',
            'value'   => 'document_base_url : "'.$app_path.'",',
         ),
	 array(
            'file'    => '../tinymce/init.js',
            'pattern' => '/content_css.*/',
            'value'   => 'content_css : "'.$app_path.'styles/style_rte.css"',
         ),
         array(
             'file'    => '../tinymce/plugins/ibrowser/config/config.inc.php',
             'pattern' => '/\'value\'\s=>\s\'.*\/img\/\'/',
             'value'   => '\'value\' => \''.$app_path.'img/\'',
         ),
	 array(
             'file'    => '../tinymce/plugins/ibrowser/config/config.inc.php',
             'pattern' => '/\'value\'\s=>\s\'.*\/img\/homepage\/\'/',
             'value'   => '\'value\' => \''.$app_path.'img/homepage/\'',
         ),
     );
    
	// check if file_exists and are writeable
	// if not return error messages
	foreach($connection_files as $file) {
	    if(!file_exists($file) || !is_writeable($file) || !is_readable($file)) {
			$errors[] = $file.' '.NOT_WRITEABLE;
		}
	}
    foreach($tinyMCE as $mceConfig) {
      if(!file_exists($mceConfig['file']) || !is_writeable($mceConfig['file']) || !is_readable($mceConfig['file'])) {
			$errors[] = $mceConfig['file'].' '.NOT_WRITEABLE;
		}
    }
	if(!empty($errors)) {
	    return $errors;
	}
	
    // modify tinymce config file
    foreach($tinyMCE as $mceConfig) {
      $fp = fopen($mceConfig['file'], 'r');
      $file_content = '';
	  while(!feof($fp)) {
		$file_content .= fread($fp, 1024);
      }
      fclose($fp);
      
      // the new content      
      $file_content = preg_replace($mceConfig['pattern'], $mceConfig['value'], $file_content);
      
      // write new content
      $fp = fopen($mceConfig['file'], 'w');
	  fwrite($fp, $file_content);
	  fclose($fp);	
    }
    
	// modify all connection files
	foreach($connection_files as $key => $file) {

		// fetch file content
		$fp = fopen($file, 'r');
		$file_content = '';
		while(!feof($fp)) {
		    $file_content .= fread($fp, 1024);
		}
		
		// define patterns to be used for replacement
		switch($key) {
		    
			// pst.php
			case 'pst':				
				$pattern['hostname'] = '/(\$hostname_pst).*;/';
				$pattern['database'] = '/(\$database_pst).*;/';
				$pattern['username'] = '/(\$username_pst).*;/';
				$pattern['password'] = '/(\$password_pst).*;/';
			break;
					
			// KT_db.php and KT_frontend_db.php
			case ('kt' || 'kt_frontend'):
			    $pattern['hostname'] = '/(\$MM_KT_db_HOSTNAME).*;/';
				$pattern['database'] = '/(\$MM_KT_db_DATABASE).*;/';
				$pattern['username'] = '/(\$MM_KT_db_USERNAME).*;/';
				$pattern['password'] = '/(\$MM_KT_db_PASSWORD).*;/';
			break;
					
		}
		// parse the file
		foreach($db_settings as $name => $value) {
		    if($name == 'database' && ($key != 'pst')) {
			    $replace      = '\1 = "mysql:'.$value.'";';
			} else {		
			    $replace      = '\1 = "'.$value.'";';
			}
			if(isset($pattern[$name])) {
			    $file_content = preg_replace($pattern[$name], $replace, $file_content, 1);
			}
		}
		// write the new content
		$fp = fopen($file, 'w');
		fwrite($fp, $file_content);
		fclose($fp);		
	}
	
	// here we always return an empty array
	return $errors;	
}




/**
* Imports a MySQL Dump into an Database
*
* @param string $file
* @access public
* @return array
*/
function import_database_file($file)
{
    // init error array
	$errors = array();
	
    // get file content
	$sql_code = file_get_contents($file);
	
	// parse file content into an array
    $sql_queries = get_query_array($sql_code, 1);
	
	// import the database
	foreach($sql_queries as $query) {
		$result = @mysql_query($query);
	    
		// if query failed return the mysql_error
		if(!$result) {
		   $errors[] = mysql_error();
		   return $errors;
		}
	}
	
	// here we return only an empty array
	return $errors;
}

/**
* Seperates all queries from a Database Dump
* into single queries
*
* @param string $sql_code
* @param integer $remove_non_sql optional, default value 0
* @param string $query_regex optional, default value ''
* @access public
* @return array
*/
function get_query_array($sql_code,$remove_non_sql=0,$query_regex='')
{	
  $sql_codeArr = explode(chr(10), $sql_code);
  
  $statementArray        = array();
  $statementArrayPointer = 0;
  
  foreach($sql_codeArr as $line => $linecontent) {
  	$is_set = 0;
  	if(stristr($linecontent, 'auto_increment')) {
  		$linecontent = eregi_replace(' default \'0\'', '', $linecontent);
  	}
  
  	if (!$remove_non_sql || (strcmp(trim($linecontent), '') &&
  	    substr(trim($linecontent), 0, 1)!='#' && substr(trim($linecontent), 0, 2)!='--'))	{
  	    // '--' is seen as comments from mysql version 3.23.49
  		$statementArray[$statementArrayPointer] .= $linecontent;
  		$is_set = 1;
  	}
  	if (substr(trim($linecontent),-1)==';')	{
  		if (isset($statementArray[$statementArrayPointer]))	{
  			if (!trim($statementArray[$statementArrayPointer]) || ($query_regex && !eregi($query_regex,trim($statementArray[$statementArrayPointer]))))	{
  				unset($statementArray[$statementArrayPointer]);
  			}
  		}
  		$statementArrayPointer++;
  	} elseif ($is_set) {
  		$statementArray[$statementArrayPointer] .= chr(10);
  	}
  }
  return $statementArray;
}

/**
* Fetches all SQL-files from the Current directory
*
* @access public
* @return array
*/
function get_database_files()
{
 	return glob('*.sql');
}


/**
* Tries to load the ionCube loader and returns the result
*
* @access public
* @return bool
*/
function load_ioncube_loader()
{
	if(extension_loaded('ionCube Loader')) {
		return true;	
	}
	
	$__oc   = strtolower(substr(php_uname(), 0, 3));
	$__ln   = '/ioncube/ioncube_loader_'.$__oc.'_'.substr(phpversion(),0,3).(($__oc=='win')?'.dll':'.so');
	$__oid  = $__id=realpath(ini_get('extension_dir'));
	$__here = dirname(__FILE__);
	if(strlen($__id)>1&&$__id[1]==':') {
		$__id   = str_replace('\\','/',substr($__id,2));
		$__here = str_replace('\\','/',substr($__here,2));
	}
	$__rd=str_repeat('/..',substr_count($__id,'/')).$__here.'/';
	$__i=strlen($__rd);
	while($__i--) {
		if($__rd[$__i]=='/') {
			$__lp=substr($__rd,0,$__i).$__ln;
			if(file_exists($__oid.$__lp)) {
				$__ln=$__lp;
				break;
			}
		}
	}
	@dl($__ln);
	
	if(extension_loaded('ionCube Loader')) {
		return true;	
	} else {
		return false;	
	}
}
?>